namespace CodeSearchEngine
{
	using System.Collections.Generic;
	using ICSharpCode.NRefactory.Ast;
	using ICSharpCode.NRefactory.Visitors;
	using Lucene.Net.Documents;

	public class GenerateLuceneDocuments : AbstractAstVisitor
	{
		public List<Document> Documents = new List<Document>();

		private readonly string file;

		public GenerateLuceneDocuments(string file)
		{
			this.file = file;
		}

		public override object VisitTypeDeclaration(TypeDeclaration typeDeclaration, object data)
		{
			AddDocument(typeDeclaration, "type", typeDeclaration.Name);
			return base.VisitTypeDeclaration(typeDeclaration, data);
		}

		public override object VisitMethodDeclaration(MethodDeclaration methodDeclaration, object data)
		{
			AddDocument(methodDeclaration, "method", methodDeclaration.Name);
			return base.VisitMethodDeclaration(methodDeclaration, data);
		}

		public override object VisitEventDeclaration(EventDeclaration eventDeclaration, object data)
		{
			AddDocument(eventDeclaration, "event", eventDeclaration.Name);
			return base.VisitEventDeclaration(eventDeclaration, data);
		}

		private void AddDocument(INode typeDeclaration, string mainAttribute, string mainAttributeValue)
		{
			Document currentDocument = new Document();

			currentDocument.Add(new Field(mainAttribute, mainAttributeValue, Field.Store.NO, Field.Index.TOKENIZED));
			currentDocument.Add(new Field("filename", file, Field.Store.YES, Field.Index.TOKENIZED));
			currentDocument.Add(new Field("column", typeDeclaration.StartLocation.X.ToString(),Field.Store.YES, Field.Index.NO));
			currentDocument.Add(new Field("line", typeDeclaration.StartLocation.Y.ToString(), Field.Store.YES, Field.Index.NO));
			
			Documents.Add(currentDocument);
		}
	}
}