namespace CodeSearchEngine
{
	using System;
	using System.Collections.Generic;
	using ICSharpCode.NRefactory;
	using ICSharpCode.NRefactory.Ast;
	using ICSharpCode.NRefactory.Visitors;
	using Lucene.Net.Analysis.Standard;
	using Lucene.Net.Documents;
	using Lucene.Net.Index;
	using Lucene.Net.QueryParsers;
	using Lucene.Net.Search;

	public class Program
	{
		const string rootDirectory = @"D:\OSS\nhibernate\src\NHibernate";

		public static void Main(string[] args)
		{
			List<Document> documents = new List<Document>();
			FileSearcherHelper.IterateAllFiles(rootDirectory, "*.cs", delegate(string file)
			{
				IParser parser = ParserFactory.CreateParser(file);
				parser.Parse();
				GenerateLuceneDocuments visitor = new GenerateLuceneDocuments(file);
				parser.CompilationUnit.AcceptVisitor(visitor, null);
				documents.AddRange(visitor.Documents);
				Console.WriteLine("{0} processed", file);
			});
			Console.WriteLine();
			Console.WriteLine("Finished parsing");
			Console.WriteLine();
			IndexWriter writer = new IndexWriter("index", new StandardAnalyzer(), true);

			foreach (Document document in documents)
			{
				writer.AddDocument(document);
			}
			writer.Close();
			
			Console.WriteLine();
			Console.WriteLine("Finished persisting search");
			Console.WriteLine();

			while (true)
			{
				string input = Console.ReadLine();

				IndexSearcher searcher = new IndexSearcher("index");
				QueryParser queryParser = new QueryParser("", new StandardAnalyzer());
				Query query = queryParser.Parse(input);
				Hits search = searcher.Search(query);
				for (int i = 0; i < search.Length(); i++)
				{
					string file = search.Doc(i).GetField("filename").StringValue();
					string line = search.Doc(i).GetField("line").StringValue();
					string column = search.Doc(i).GetField("column").StringValue();
					Console.WriteLine("{0} line: {1}, column: {2}",
						file,
						line,
						column);

				}
			}
		}
	}


	public class PrintResultsVisitor : AbstractAstVisitor
	{
		public override object VisitMethodDeclaration(MethodDeclaration methodDeclaration, object data)
		{
			Console.WriteLine("\t{0}", methodDeclaration.Name);
			return null;
		}
	}

}